@@ -4,20 +4,6 @@ from .base import ItemFinder, stack
 
 class DoxygenTypeSubItemFinder(ItemFinder):
 
-    def find(self, matcher_stack):
-
-        compounds = self.data_object.get_compound()
-
-        results = []
-
-        for compound in compounds:
-
-            if matcher_stack.match("compound", compound):
-                compound_finder = self.item_finder_factory.create_finder(compound)
-                results.extend(compound_finder.find(matcher_stack))
-
-        return results
-
     def filter_(self, ancestors, filter_, matches):
         """Find nodes which match the filter. Doesn't test this node, only its children"""
 
@@ -39,34 +25,6 @@ class CompoundTypeSubItemFinder(ItemFinder):
         self.filter_factory = filter_factory
         self.compound_parser = compound_parser
 
-    def find(self, matcher_stack):
-
-        members = self.data_object.get_member()
-
-        member_results = []
-
-        for member in members:
-            if matcher_stack.match("member", member):
-                member_finder = self.item_finder_factory.create_finder(member)
-                member_results.extend(member_finder.find(matcher_stack))
-
-        results = []
-
-        # If there are members in this compound that match the criteria
-        # then load up the file for this compound and get the member data objects
-        if member_results:
-
-            file_data = self.compound_parser.parse(self.data_object.refid)
-            finder = self.item_finder_factory.create_finder(file_data)
-
-            for member_data in member_results:
-                results.extend(finder.find(matcher_stack))
-
-        elif matcher_stack.full_match("compound", self.data_object):
-            results.append(self.data_object)
-
-        return results
-
     def filter_(self, ancestors, filter_, matches):
         """Finds nodes which match the filter and continues checks to children
 
@@ -79,7 +37,7 @@ class CompoundTypeSubItemFinder(ItemFinder):
 
         # Match against compound object
         if filter_.allow(node_stack):
-            matches.append(self.data_object)
+            matches.append(node_stack)
 
         # Descend to member children
         members = self.data_object.get_member()
@@ -97,8 +55,8 @@ class CompoundTypeSubItemFinder(ItemFinder):
             file_data = self.compound_parser.parse(self.data_object.refid)
             finder = self.item_finder_factory.create_finder(file_data)
 
-            for member_data in member_matches:
-                ref_filter = self.filter_factory.create_id_filter('memberdef', member_data.refid)
+            for member_stack in member_matches:
+                ref_filter = self.filter_factory.create_id_filter('memberdef', member_stack[0].refid)
                 finder.filter_(node_stack, ref_filter, matches)
 
         else:
@@ -112,19 +70,10 @@ class CompoundTypeSubItemFinder(ItemFinder):
 
 class MemberTypeSubItemFinder(ItemFinder):
 
-    def find(self, matcher_stack):
-
-        if matcher_stack.full_match("member", self.data_object):
-            return [self.data_object]
-        else:
-            return []
-
     def filter_(self, ancestors, filter_, matches):
 
         node_stack = stack(self.data_object, ancestors)
 
         # Match against member object
         if filter_.allow(node_stack):
-            matches.append(self.data_object)
-
-
+            matches.append(node_stack)
