@@ -1,6 +1,8 @@
 
 from .base import Renderer
 from .index import CompoundRenderer
+import six
+
 
 class DoxygenTypeSubRenderer(Renderer):
 
@@ -17,47 +19,47 @@ class CompoundDefTypeSubRenderer(Renderer):
     # here and the titles for the SectionDefTypeSubRenderer but we don't want the repetition of
     # having two lists in case they fall out of sync
     sections = [
-                ("user-defined", "User Defined"),
-                ("public-type", "Public Types"),
-                ("public-func", "Public Functions"),
-                ("public-attrib", "Public Members"),
-                ("public-slot", "Public Slots"),
-                ("signal", "Signal"),
-                ("dcop-func",  "DCOP Function"),
-                ("property",  "Property"),
-                ("event",  "Event"),
-                ("public-static-func", "Public Static Functions"),
-                ("public-static-attrib", "Public Static Attributes"),
-                ("protected-type",  "Protected Types"),
-                ("protected-func",  "Protected Functions"),
-                ("protected-attrib",  "Protected Attributes"),
-                ("protected-slot",  "Protected Slots"),
-                ("protected-static-func",  "Protected Static Functions"),
-                ("protected-static-attrib",  "Protected Static Attributes"),
-                ("package-type",  "Package Types"),
-                ("package-func", "Package Functions"),
-                ("package-attrib", "Package Attributes"),
-                ("package-static-func", "Package Static Functions"),
-                ("package-static-attrib", "Package Static Attributes"),
-                ("private-type", "Private Types"),
-                ("private-func", "Private Functions"),
-                ("private-attrib", "Private Members"),
-                ("private-slot",  "Private Slots"),
-                ("private-static-func", "Private Static Functions"),
-                ("private-static-attrib",  "Private Static Attributes"),
-                ("friend",  "Friends"),
-                ("related",  "Related"),
-                ("define",  "Defines"),
-                ("prototype",  "Prototypes"),
-                ("typedef",  "Typedefs"),
-                ("enum",  "Enums"),
-                ("func",  "Functions"),
-                ("var",  "Variables"),
-                ]
+        ("user-defined", "User Defined"),
+        ("public-type", "Public Types"),
+        ("public-func", "Public Functions"),
+        ("public-attrib", "Public Members"),
+        ("public-slot", "Public Slots"),
+        ("signal", "Signal"),
+        ("dcop-func",  "DCOP Function"),
+        ("property",  "Property"),
+        ("event",  "Event"),
+        ("public-static-func", "Public Static Functions"),
+        ("public-static-attrib", "Public Static Attributes"),
+        ("protected-type",  "Protected Types"),
+        ("protected-func",  "Protected Functions"),
+        ("protected-attrib",  "Protected Attributes"),
+        ("protected-slot",  "Protected Slots"),
+        ("protected-static-func",  "Protected Static Functions"),
+        ("protected-static-attrib",  "Protected Static Attributes"),
+        ("package-type",  "Package Types"),
+        ("package-func", "Package Functions"),
+        ("package-attrib", "Package Attributes"),
+        ("package-static-func", "Package Static Functions"),
+        ("package-static-attrib", "Package Static Attributes"),
+        ("private-type", "Private Types"),
+        ("private-func", "Private Functions"),
+        ("private-attrib", "Private Members"),
+        ("private-slot",  "Private Slots"),
+        ("private-static-func", "Private Static Functions"),
+        ("private-static-attrib",  "Private Static Attributes"),
+        ("friend",  "Friends"),
+        ("related",  "Related"),
+        ("define",  "Defines"),
+        ("prototype",  "Prototypes"),
+        ("typedef",  "Typedefs"),
+        ("enum",  "Enums"),
+        ("func",  "Functions"),
+        ("var",  "Variables"),
+    ]
 
     def render(self):
 
-        nodelist = []    
+        nodelist = []
 
         if self.data_object.briefdescription:
             context = self.context.create_child_context(self.data_object.briefdescription)
@@ -223,6 +225,58 @@ class MemberDefTypeSubRenderer(Renderer):
         return nodes
 
 
+def get_param_decl(param):
+
+    param_type = []
+    for p in param.type_.content_:
+        value = p.value
+        if not isinstance(value, six.text_type):
+            value = value.valueOf_
+        param_type.append(value)
+    param_type = ' '.join(param_type)
+    param_name = param.declname if param.declname else param.defname
+    if not param_name:
+        param_decl = param_type
+    elif '(*)' in param_type:
+        param_decl = param_type.replace('(*)', '(*' + param_name + ')')
+    elif '(&)' in param_type:
+        param_decl = param_type.replace('(&)', '(&' + param_name + ')')
+    else:
+        param_decl = param_type + ' ' + param_name
+    if param.array:
+        param_decl += param.array
+
+    return param_decl
+
+
+def get_definition_without_template_args(data_object):
+    """
+    Return data_object.definition removing any template arguments from the class name in the member
+    function.  Otherwise links to classes defined in the same template are not generated correctly.
+
+    For example in 'Result<T> A< B<C> >::f' we want to remove the '< B<C> >' part.
+    """
+    definition = data_object.definition
+    qual_name = '::' + data_object.name
+    if definition.endswith(qual_name):
+        qual_name_start = len(definition) - len(qual_name)
+        pos = qual_name_start - 1
+        if definition[pos] == '>':
+            bracket_count = 0
+            # Iterate back through the characters of the definition counting matching braces and
+            # then remove all braces and everything between
+            while pos > 0:
+                if definition[pos] == '>':
+                    bracket_count += 1
+                elif definition[pos] == '<':
+                    bracket_count -= 1
+                    if bracket_count == 0:
+                        definition = definition[:pos] + definition[qual_name_start:]
+                        break
+                pos -= 1
+    return definition
+
+
 class FuncMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
 
     def update_signature(self, signode):
@@ -247,19 +301,13 @@ class FuncMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
 
     def render(self):
         # Get full function signature for the domain directive.
-        params = []
+        param_list = []
         for param in self.data_object.param:
             param = self.context.mask_factory.mask(param)
-            param_type = []
-            for p in param.type_.content_:
-                value = p.value
-                if not isinstance(value, unicode):
-                    value = value.valueOf_
-                param_type.append(value)
-            param_type = ' '.join(param_type)
-            param_name = param.declname if param.declname else param.defname
-            params.append(param_type if not param_name else param_type + ' ' + param_name)
-        signature = '{0}({1})'.format(self.data_object.definition, ', '.join(params))
+            param_decl = get_param_decl(param)
+            param_list.append(param_decl)
+        signature = '{0}({1})'.format(get_definition_without_template_args(self.data_object),
+                                      ', '.join(param_list))
         # Remove 'virtual' keyword as Sphinx 1.2 doesn't support virtual functions.
         virtual = 'virtual '
         if signature.startswith(virtual):
@@ -304,8 +352,8 @@ class EnumMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
 
     def declaration(self):
 
-        # Sphinx requires a name to be a valid identifier, so replace anonymous enum name of the form @id
-        # generated by Doxygen with "anonymous".
+        # Sphinx requires a name to be a valid identifier, so replace anonymous enum name of the
+        # form @id generated by Doxygen with "anonymous".
         name = self.get_fully_qualified_name()
         return name.replace("@", "__anonymous") if self.data_object.name.startswith("@") else name
 
@@ -328,9 +376,14 @@ class EnumMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
         return description_nodes
 
     def update_signature(self, signode):
-        # Replace "type" with "enum" in the signature. This is needed because Sphinx cpp domain doesn't have an enum
-        # directive and we use a type directive instead.
-        signode.children[0][0] = self.node_factory.Text("enum ")
+        first_node = signode.children[0]
+        if isinstance(first_node, self.node_factory.desc_annotation):
+            # Replace "type" with "enum" in the signature. This is needed because older versions of
+            # Sphinx cpp domain didn't have an enum directive and we use a type directive instead.
+            first_node[0] = self.node_factory.Text("enum ")
+        else:
+            # If there is no "type" annotation, insert "enum".
+            first_node.insert(0, self.node_factory.desc_annotation("enum ", "enum "))
         if self.data_object.name.startswith("@"):
             signode.children[1][0] = self.node_factory.strong(text="[anonymous]")
 
@@ -338,7 +391,7 @@ class EnumMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
 class TypedefMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
 
     def declaration(self):
-        decl = self.data_object.definition
+        decl = get_definition_without_template_args(self.data_object)
         typedef = "typedef "
         if decl.startswith(typedef):
             decl = decl[len(typedef):]
@@ -348,7 +401,7 @@ class TypedefMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
 class VariableMemberDefTypeSubRenderer(MemberDefTypeSubRenderer):
 
     def declaration(self):
-        decl = self.data_object.definition
+        decl = get_definition_without_template_args(self.data_object)
         enum = 'enum '
         return decl[len(enum):] if decl.startswith(enum) else decl
 
@@ -363,8 +416,8 @@ class EnumvalueTypeSubRenderer(MemberDefTypeSubRenderer):
         return 'enumvalue'
 
     def update_signature(self, signode):
-        # Remove "class" from the signature. This is needed because Sphinx cpp domain doesn't have an enum value
-        # directive and we use a class directive instead.
+        # Remove "class" from the signature. This is needed because Sphinx cpp domain doesn't have
+        # an enum value directive and we use a class directive instead.
         signode.children.pop(0)
         initializer = self.data_object.initializer
         if initializer:
@@ -416,8 +469,7 @@ class ParamTypeSubRenderer(Renderer):
             *args
             ):
 
-        Renderer.__init__( self, *args )
-
+        Renderer.__init__(self, *args)
         self.output_defname = output_defname
 
     def render(self):
@@ -428,17 +480,28 @@ class ParamTypeSubRenderer(Renderer):
         if self.data_object.type_:
             context = self.context.create_child_context(self.data_object.type_)
             renderer = self.renderer_factory.create_renderer(context)
-            nodelist.extend(renderer.render())
+            type_nodes = renderer.render()
+            # Render keywords as annotations for consistency with the cpp domain.
+            if len(type_nodes) > 0:
+                first_node = type_nodes[0]
+                for keyword in ['typename', 'class']:
+                    if first_node.startswith(keyword + ' '):
+                        type_nodes[0] = self.node_factory.Text(first_node.replace(keyword, '', 1))
+                        type_nodes.insert(0, self.node_factory.desc_annotation(keyword, keyword))
+                        break
+            nodelist.extend(type_nodes)
 
         # Parameter name
         if self.data_object.declname:
-            if nodelist: nodelist.append(self.node_factory.Text(" "))
+            if nodelist:
+                nodelist.append(self.node_factory.Text(" "))
             nodelist.append(self.node_factory.emphasis(text=self.data_object.declname))
 
         elif self.output_defname and self.data_object.defname:
             # We only want to output the definition name (from the cpp file) if the declaration name
             # (from header file) isn't present
-            if nodelist: nodelist.append(self.node_factory.Text(" "))
+            if nodelist:
+                nodelist.append(self.node_factory.Text(" "))
             nodelist.append(self.node_factory.emphasis(text=self.data_object.defname))
 
         # array information
@@ -455,7 +518,6 @@ class ParamTypeSubRenderer(Renderer):
         return nodelist
 
 
-
 class DocRefTextTypeSubRenderer(Renderer):
 
     def render(self):
@@ -474,16 +536,16 @@ class DocRefTextTypeSubRenderer(Renderer):
 
         refid = "%s%s" % (self.project_info.name(), self.data_object.refid)
         nodelist = [
-                self.node_factory.pending_xref(
-                    "",
-                    reftype="ref",
-                    refdomain="std",
-                    refexplicit=True,
-                    refid=refid, 
-                    reftarget=refid,
-                    *nodelist
-                    )
-                ]
+            self.node_factory.pending_xref(
+                "",
+                reftype="ref",
+                refdomain="std",
+                refexplicit=True,
+                refid=refid,
+                reftarget=refid,
+                *nodelist
+            )
+        ]
 
         return nodelist
 
@@ -538,13 +600,14 @@ class DocImageTypeSubRenderer(Renderer):
     def render(self):
 
         path_to_image = self.project_info.sphinx_abs_path_to_file(
-                self.data_object.name
-                )
+            self.data_object.name
+        )
 
-        options = { "uri" : path_to_image }
+        options = {"uri": path_to_image}
 
         return [self.node_factory.image("", **options)]
 
+
 class DocMarkupTypeSubRenderer(Renderer):
 
     def __init__(
@@ -553,8 +616,7 @@ class DocMarkupTypeSubRenderer(Renderer):
             *args
             ):
 
-        Renderer.__init__( self, *args )
-
+        Renderer.__init__(self, *args)
         self.creator = creator
 
     def render(self):
@@ -573,11 +635,11 @@ class DocParamListTypeSubRenderer(Renderer):
     """Parameter/Exception documentation"""
 
     lookup = {
-            "param" : "Parameters",
-            "exception" : "Exceptions",
-            "templateparam" : "Templates",
-            "retval" : "Return Value",
-            }
+        "param": "Parameters",
+        "exception": "Exceptions",
+        "templateparam": "Template Parameters",
+        "retval": "Return Value",
+    }
 
     def render(self):
 
@@ -588,10 +650,11 @@ class DocParamListTypeSubRenderer(Renderer):
             nodelist.extend(renderer.render())
 
         # Fild list entry
-        nodelist_list = self.node_factory.bullet_list("", classes=["breatheparameterlist"], *nodelist)
+        nodelist_list = self.node_factory.bullet_list("", classes=["breatheparameterlist"],
+                                                      *nodelist)
 
         term_text = self.lookup[self.data_object.kind]
-        term = self.node_factory.term("", "", self.node_factory.strong( "", term_text ) )
+        term = self.node_factory.term("", "", self.node_factory.strong("", term_text))
         definition = self.node_factory.definition('', nodelist_list)
 
         return [self.node_factory.definition_list_item('', term, definition)]
@@ -608,7 +671,7 @@ class DocParamListItemSubRenderer(Renderer):
             renderer = self.renderer_factory.create_renderer(context)
             nodelist.extend(renderer.render())
 
-        term = self.node_factory.literal("","", *nodelist)
+        term = self.node_factory.literal("", "", *nodelist)
 
         separator = self.node_factory.Text(" - ")
 
@@ -621,6 +684,7 @@ class DocParamListItemSubRenderer(Renderer):
 
         return [self.node_factory.list_item("", term, separator, *nodelist)]
 
+
 class DocParamNameListSubRenderer(Renderer):
     """ Parameter Name Renderer """
 
@@ -634,6 +698,7 @@ class DocParamNameListSubRenderer(Renderer):
 
         return nodelist
 
+
 class DocParamNameSubRenderer(Renderer):
 
     def render(self):
@@ -646,6 +711,7 @@ class DocParamNameSubRenderer(Renderer):
 
         return nodelist
 
+
 class DocSect1TypeSubRenderer(Renderer):
 
     def render(self):
@@ -660,7 +726,7 @@ class DocSimpleSectTypeSubRenderer(Renderer):
 
         text = self.node_factory.Text(self.data_object.kind.capitalize())
 
-        return [self.node_factory.strong( "", text )]
+        return [self.node_factory.strong("", text)]
 
     def render(self):
 
@@ -683,7 +749,7 @@ class ParDocSimpleSectTypeSubRenderer(DocSimpleSectTypeSubRenderer):
         context = self.context.create_child_context(self.data_object.title)
         renderer = self.renderer_factory.create_renderer(context)
 
-        return [self.node_factory.strong( "", *renderer.render() )]
+        return [self.node_factory.strong("", *renderer.render())]
 
 
 class DocTitleTypeSubRenderer(Renderer):
@@ -746,7 +812,6 @@ class ListingTypeSubRenderer(Renderer):
 
     def render(self):
 
-        lines = []
         nodelist = []
         for i, item in enumerate(self.data_object.codeline):
             # Put new lines between the lines. There must be a more pythonic way of doing this
@@ -759,13 +824,14 @@ class ListingTypeSubRenderer(Renderer):
         # Add blank string at the start otherwise for some reason it renders
         # the pending_xref tags around the kind in plain text
         block = self.node_factory.literal_block(
-                "",
-                "",
-                *nodelist
-                )
+            "",
+            "",
+            *nodelist
+        )
 
         return [block]
 
+
 class CodeLineTypeSubRenderer(Renderer):
 
     def render(self):
@@ -778,6 +844,7 @@ class CodeLineTypeSubRenderer(Renderer):
 
         return nodelist
 
+
 class HighlightTypeSubRenderer(Renderer):
 
     def render(self):
@@ -790,6 +857,7 @@ class HighlightTypeSubRenderer(Renderer):
 
         return nodelist
 
+
 class TemplateParamListRenderer(Renderer):
 
     def render(self):
@@ -805,6 +873,7 @@ class TemplateParamListRenderer(Renderer):
 
         return nodelist
 
+
 class IncTypeSubRenderer(Renderer):
 
     def render(self):
